import 'dart:async';

import 'package:flutter/services.dart';

class BrazePlugin {
  static const MethodChannel _channel = const MethodChannel('braze_plugin');

  void changeUser(String userId) {
    final Map<String, dynamic> params = <String, dynamic>{
      "userId" : userId
    };
    _channel.invokeMethod('changeUser', params);
  }

  void logCustomEvent(String eventName) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName" : eventName
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  void logCustomEventWithProperties(String eventName,
      Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName" : eventName,
      "properties" : properties
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  void logPurchase(String productId, String currencyCode, double price,
      int quantity) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId" : productId,
      "currencyCode" : currencyCode,
      "price" : price,
      "quantity" : quantity
    };
    _channel.invokeMethod('logPurchase', params);
  }

  void logPurchaseWithProperties(String productId, String currencyCode,
      double price, int quantity, Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId" : productId,
      "currencyCode" : currencyCode,
      "price" : price,
      "quantity" : quantity,
      "properties" : properties
    };
    _channel.invokeMethod('logPurchase', params);
  }

  void addToCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value
    };
    _channel.invokeMethod('addToCustomAttributeArray', params);
  }

  void removeFromCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value
    };
    _channel.invokeMethod('removeFromCustomAttributeArray', params);
  }

  void setStringCustomUserAttribute(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value
    };
    _channel.invokeMethod('setStringCustomUserAttribute', params);
  }

  void setDoubleCustomUserAttribute(String key, double value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value
    };
    _channel.invokeMethod('setDoubleCustomUserAttribute', params);
  }

  void setBoolCustomUserAttribute(String key, bool value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value
    };
    _channel.invokeMethod('setBoolCustomUserAttribute', params);
  }

  void setIntCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value
    };
    _channel.invokeMethod('setIntCustomUserAttribute', params);
  }

  void incrementCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value
    };
    _channel.invokeMethod('incrementCustomUserAttribute', params);
  }

  void setLocationCustomAttribute(String key, double lat, double long) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'lat' : lat,
      'long' : long
    };
    _channel.invokeMethod('setLocationCustomAttribute', params);
  }

  void setDateCustomUserAttribute(String key, DateTime value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key' : key,
      'value' : value.millisecondsSinceEpoch ~/ 1000
    };
    _channel.invokeMethod('setDateCustomUserAttribute', params);
  }

  void unsetCustomUserAttribute(String key) {
    callStringMethod('unsetCustomUserAttribute', 'key', key);
  }

  void setFirstName(String firstName) {
    callStringMethod('setFirstName', 'firstName', firstName);
  }

  void setLastName(String lastName) {
    callStringMethod('setLastName', 'lastName', lastName);
  }

  void setEmail(String email) {
    callStringMethod('setEmail', 'email', email);
  }

  void setDateOfBirth(int year, int month, int day) {
    final Map<String, dynamic> params = <String, dynamic>{
      'year' : year,
      'month' : month,
      'day' : day
    };
    _channel.invokeMethod('setDateOfBirth', params);
  }

  void setGender(String gender) {
    callStringMethod('setGender', 'gender', gender);
  }

  void setLanguage(String language) {
    callStringMethod('setLanguage', 'language', language);
  }

  void setCountry(String country) {
    callStringMethod('setCountry', 'country', country);
  }

  void setHomeCity(String homeCity) {
    callStringMethod('setHomeCity', 'homeCity', homeCity);
  }

  void setPhoneNumber(String phoneNumber) {
    callStringMethod('setPhoneNumber', 'phoneNumber', phoneNumber);
  }

  void setAvatarImageUrl(String avatarImageUrl) {
    callStringMethod('setAvatarImageUrl', 'avatarImageUrl', avatarImageUrl);
  }

  void setAttributionData(String network, String campaign, String adGroup,
      String creative) {
    final Map<String, dynamic> params = <String, dynamic>{
      'network' : network,
      'campaign' : campaign,
      'adGroup' : adGroup,
      'creative' : creative
    };
    _channel.invokeMethod('setAttributionData', params);
  }

  void registerPushToken(String pushToken) {
    callStringMethod('registerPushToken', 'pushToken', pushToken);
  }

  void requestImmediateDataFlush() {
    _channel.invokeMethod('requestImmediateDataFlush');
  }

  void wipeData() {
    _channel.invokeMethod('wipeData');
  }

  void enableSDK() {
    _channel.invokeMethod('enableSDK');
  }

  void disableSDK() {
    _channel.invokeMethod('disableSDK');
  }

  void setPushNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type' : type.toString()
    };
    _channel.invokeMethod('setPushNotificationSubscriptionType', params);
  }

  void setEmailNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type' : type.toString()
    };
    _channel.invokeMethod('setEmailNotificationSubscriptionType', params);
  }

  void callStringMethod(String methodName, String paramName, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      paramName : value
    };
    _channel.invokeMethod(methodName, params);
  }
}

enum SubscriptionType {
  subscribed, unsubscribed, opted_in
}
